﻿using UnityEngine;
using System.Collections;

public enum AnimationBam {
	AnkhegGeneric1,
	AnkhegGeneric2,
	AnkhegGeneric3,
	CharacterGeneric1,
	CharacterGeneric2,
	CharacterGeneric3,
	CharacterCast,
	CharacterAttack1,
	CharacterAttack2,
	CharacterAttack3,
	CharacterAttack4,
	CharacterAttack5,
	CharacterAttack6,
	CharacterAttack7,
	CharacterAttack8,
	CharacterAttack9,
	CharacterBow,
	CharacterXBow,
	CharacterSling,
	IWDAttack1,
	IWDAttack2,
	IWDAttack3,
	IWDAttack4,
	IWDAwake,
	IWDCast,
	IWDConjure,
	IWDDamage,
	IWDDie,
	IWDHeadTurn,
	IWDReady,
	IWDSleep,
	IWDTwitch,
	IWDWalk,
}

public class AnimationInfo {
	public string Name;
	public int FrameCount;
	public int CurrentFrame = 0;
	public AnimationInfo(string name, int frameCount) {
		Name = name;
		FrameCount = frameCount;
	}
	
	public float Progress() {
		return (float)CurrentFrame / FrameCount;
	}
	
	public void Step() {
		CurrentFrame++;
	}
	public bool IsComplete() {
		return CurrentFrame == FrameCount;
	}
	public void Reset() {
    	CurrentFrame = 0;
	}
	
}
public class RenderBG : MonoBehaviour {
	public GameObject viewCamera;
	public GameObject target;
	
	public bool Mirrored = false;
	public AnimationBam AnimationBam;
	public string BamName;
	public Shader RenderShader;
	
	private Vector3[] positions;

	private int skipFrames;
	
	private AnimationInfo[] Animations;
	/*= new AnimationInfo[] {
		new AnimationInfo("attack1",   30),
		new AnimationInfo("awake",     30),
		new AnimationInfo("cast",      30),
		new AnimationInfo("conjure",   30),
		new AnimationInfo("damage",    30),
		new AnimationInfo("die",       30),
		new AnimationInfo("head_turn", 30),
		new AnimationInfo("ready",     30),
		new AnimationInfo("shoot",     30),
		new AnimationInfo("twitch",    30),
		new AnimationInfo("walk",      30),
		new AnimationInfo("attack2",   30),
		new AnimationInfo("attack3",   30), 
		new AnimationInfo("attack4",   30), 
		new AnimationInfo("sleep",     30),
	};*/
	private int currentAnimation = 0;
	private int currentPosition = 0;
	private int startPosition = 0;
	private int endPosition = 0;
	private int numPositions = 0;
	private bool takeScreenshots;
	//private RenderTexture rt;
	private Texture2D screenShot;
	private string bamdFrames;
	private int frameCounter;
	private string bamdSequences;
	private string bamdExtension;
	
	// Use this for initialization
	void Start () {
		float positionScale = 360.0f;

		switch(AnimationBam) {
			case AnimationBam.CharacterGeneric1:
				numPositions = 9;
				currentPosition = startPosition = 0;
				endPosition = 9;
				bamdExtension = "G1";

				Animations = new AnimationInfo[] {
					new AnimationInfo("walk",       10),
						new AnimationInfo("ready_1h",   12),
						new AnimationInfo("head_turn1", 65),
						new AnimationInfo("ready_2h",   26),
						new AnimationInfo("damage",     9),
						new AnimationInfo("die",        19),
						new AnimationInfo("twitch",     6),
						new AnimationInfo("head_turn2", 38),
						new AnimationInfo("head_turn3", 46),
						new AnimationInfo("awake",      15),
						new AnimationInfo("sleep",      15),
				};
				break;
			case AnimationBam.CharacterAttack1:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 8;
				bamdExtension = "A1";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack_1h_slash", 15),
				};
				break;
			case AnimationBam.CharacterAttack2:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "A2";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack_2h_slash", 15),
				};
				break;
			case AnimationBam.CharacterAttack3:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "A3";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack_1h_backslash", 15),
				};
				break;
			case AnimationBam.CharacterAttack4:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 8;
				bamdExtension = "A4";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack_2h_backslash", 15),
				};
				break;
			case AnimationBam.CharacterAttack5:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 8;
				bamdExtension = "A5";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack_1h_jab", 15),
				};
				break;
			case AnimationBam.CharacterAttack6:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "A6";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack_2h_jab", 15),
				};
				break;
			case AnimationBam.CharacterAttack7:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "A7";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack_bh_slash", 15),
				};
				break;
			case AnimationBam.CharacterAttack8:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "A8";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack_bh_backslash", 15),
				};
				break;
			case AnimationBam.CharacterAttack9:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "A9";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack_bh_jab", 15),
				};
				break;
			case AnimationBam.CharacterBow:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "SA";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack_bow", 15),
				};
				break;
			case AnimationBam.CharacterXBow:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "SX";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack_xbow", 15),
				};
				break;
			case AnimationBam.CharacterSling:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "SS";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack_sling", 15),
				};
				break;
			case AnimationBam.AnkhegGeneric1:
			numPositions = 9;
				currentPosition = startPosition = 0;
				endPosition = 9;
				bamdExtension = "G1";
				positionScale = 180.0f;

				Animations = new AnimationInfo[] {
					new AnimationInfo("walk",   1),
						new AnimationInfo("die1",   30),
						new AnimationInfo("die2",   30),
						new AnimationInfo("ready2", 50),
				};
				break;
			case AnimationBam.AnkhegGeneric2:
				numPositions = 9;
				currentPosition = startPosition = 0;
				endPosition = 9;
				positionScale = 180.0f;
				bamdExtension = "G2";

				Animations = new AnimationInfo[] {
					new AnimationInfo("ready1",  50),
						new AnimationInfo("emerge",   30),
						new AnimationInfo("hide",     30),
				};
				break;
			case AnimationBam.AnkhegGeneric3:
				numPositions = 9;
				currentPosition = startPosition = 0;
				endPosition = 9;
				positionScale = 180.0f;
				bamdExtension = "G3";

				Animations = new AnimationInfo[] {
					new AnimationInfo("attack1",   30),
						new AnimationInfo("attack2",   40),
				};
				break;
			case AnimationBam.IWDAttack1:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "A1";
				Animations = new AnimationInfo[] {new AnimationInfo("attack1", 30)};
				break;
			case AnimationBam.IWDAttack2:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "A2";
				Animations = new AnimationInfo[] {new AnimationInfo("attack2", 30)};
				break;
			case AnimationBam.IWDAttack3:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "A3";
				Animations = new AnimationInfo[] {new AnimationInfo("attack3", 30)};
				break;
			case AnimationBam.IWDAttack4:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "A4";
				Animations = new AnimationInfo[] {new AnimationInfo("attack4", 30)};
				break;
			case AnimationBam.IWDAwake:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "AW";
				Animations = new AnimationInfo[] {new AnimationInfo("awake", 30)};
				break;
			case AnimationBam.IWDCast:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "CA";
				Animations = new AnimationInfo[] {new AnimationInfo("cast", 30)};
				break;
			case AnimationBam.IWDConjure:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "SP";
				Animations = new AnimationInfo[] {new AnimationInfo("conjure", 30)};
				break;
			case AnimationBam.IWDDamage:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "GH";
				Animations = new AnimationInfo[] {new AnimationInfo("damage", 30)};
				break;
			case AnimationBam.IWDDie:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "DE";
				Animations = new AnimationInfo[] {new AnimationInfo("die1", 30)};
				break;
			case AnimationBam.IWDHeadTurn:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "SD";
				Animations = new AnimationInfo[] {new AnimationInfo("head_turn", 30)};
				break;
			case AnimationBam.IWDReady:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "SC";
				Animations = new AnimationInfo[] {new AnimationInfo("ready", 50)};
				break;
			case AnimationBam.IWDSleep:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "SL";
				Animations = new AnimationInfo[] {new AnimationInfo("sleep", 1)};
				break;
			case AnimationBam.IWDTwitch:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "TW";
				Animations = new AnimationInfo[] {new AnimationInfo("twitch", 30)};
				break;
			case AnimationBam.IWDWalk:
				numPositions = 8;
				currentPosition = startPosition = 0;
				endPosition = 5;
				bamdExtension = "WK";
				Animations = new AnimationInfo[] {new AnimationInfo("walk", 50)};
				break;

		}
		
		if ( Mirrored ) {
			bamdExtension = bamdExtension + "E";
			currentPosition = startPosition = endPosition;
			endPosition = numPositions;
		}
		positions = new Vector3[numPositions];
		
		
		for(int i = 0; i < positions.Length; i++) {
			float degrees = -1.0f *  (float)i / numPositions * positionScale;
			float x = 5 * Mathf.Sin ( degrees * Mathf.PI/ 180.0f);
			float y = 5;
			float z = 5 * Mathf.Cos (degrees * Mathf.PI/ 180.0f);
			positions[i] = new Vector3(x, y, z);
		}

		takeScreenshots = true;

		currentAnimation = 0;
		AnimationInfo ci = Animations[currentAnimation];
		var anim = target.GetComponent<Animator>();
		anim.speed = 0;
		anim.Play (ci.Name, -1, ci.Progress ());

		screenShot = new Texture2D(resWidth, resHeight, TextureFormat.RGB24, false);
		skipFrames = 1;

		if ( startPosition > 0 ) {
			for( int i = 0; i < startPosition; i++ ) {
				bamdSequences += "sequence f00000\n";
			}
		}

	}
	
	public int resWidth = 512; 
	public int resHeight = 512;
	
	
	public static string ScreenShotName(string animation, int camera, int frame) {
		return string.Format("{0}/../../RenderOut/anim_{1}_dir_{2}_frame_{3:D5}.png", 
		                     Application.dataPath, 
		                     animation, 
		                     camera, frame);
	}
	
	void Update() {
		if (currentAnimation >= Animations.Length) {
			return;
		}
		AnimationInfo ci = Animations[currentAnimation];
		var anim = target.GetComponent<Animator>();

		anim.Play (ci.Name, -1, ci.Progress()); 

		if ( ci.IsComplete () ) {
			bamdSequences += "sequence ";
			for(int i = 0; i < ci.CurrentFrame; i++) {
				bamdSequences += string.Format("f{0:D5} ", frameCounter - ci.CurrentFrame + i);
			}
			bamdSequences += "\n";

			Debug.Log(string.Format("Current POsition: {0}", currentPosition));
			currentPosition++;
			ci.Reset();
		} 
		if ( currentPosition == endPosition ) {
			if ( currentAnimation < Animations.Length ) {
				currentAnimation++;
			}
			if ( startPosition > 0 && currentAnimation != Animations.Length ) {
				for( int i = 0; i < startPosition; i++ ) {
					bamdSequences += "sequence f00000\n";
				}
			}
			if ( endPosition < numPositions ) {
				for( int i = endPosition; i < numPositions; i++ ) {
					bamdSequences += "sequence f00000\n";
				}
			}
			if ( currentAnimation == Animations.Length ) {
				string all_text = bamdFrames + "\n\n" + bamdSequences;
				string fileName = string.Format("{0}/../../RenderOut/{1}{2}.bamd", Application.dataPath, BamName, bamdExtension);
				System.IO.File.WriteAllText(fileName, all_text);
				Debug.Log("Output bamd");
			}

			currentPosition = startPosition;
		}
		camera.transform.position = positions[currentPosition];
		camera.transform.LookAt (target.transform);
	}
	
	
	void OnPostRender() {
		if (currentAnimation >= Animations.Length) {
			return;
		}
		if ( skipFrames > 0 ) {
			skipFrames--;
		}
		AnimationInfo ci = Animations[currentAnimation];
		if ( takeScreenshots && currentAnimation < Animations.Length && skipFrames == 0 )
		{
			
			//RenderTexture.active = rt;

			screenShot.ReadPixels(new Rect(0, 0, resWidth, resHeight), 0, 0, false);
			screenShot.Apply ();
			byte[] bytes = screenShot.EncodeToPNG();
			string filename = ScreenShotName(ci.Name, currentPosition, ci.CurrentFrame);
			System.IO.File.WriteAllBytes(filename, bytes);
			//Debug.Log(string.Format("Took screenshot to: {0}", filename));
			bamdFrames += string.Format("frame f{0:D5} \"anim_{1}_dir_{2}_frame_{3:D5}.png\" {4} {5}\n", frameCounter, ci.Name, currentPosition, ci.CurrentFrame, resWidth/2, resHeight/2);
			frameCounter++;
		}
		ci.Step ();
	}
}
