#!/bin/bash
set -e
set -x
shopt -s nullglob


# This took the shadow render from unity, and changed the shadow entry to be a specific color. There is only one palette entry for shadows and we need to force it to this color
convert_shadows() {
	rm -rf output/Shadow
	mkdir -p output/Shadow
	for i in Shadow/*.png; do
		echo $i
		convert $i -fuzz 2% -transparent 'rgb(0,255,0)'  -alpha extract -transparent black  -background 'rgb(0,255,255)' -alpha shape output/Shadow/`basename $i`
	done
}

# Took the creature render converted the color keyd green to alpha
convert_frames() {
	rm -rf output/Base
	mkdir -p output/Base
	for i in Mesh/*.png; do
		echo $i
		convert $i -fuzz 2% -transparent 'rgb(0,255,0)'  -alpha on output/Base/`basename $i`
	done
}

# Composite the creature on top of the shadow
overlay_frames_on_shadows() {
	rm -rf output/Combined
	mkdir -p output/Combined
	for i in output/Base/*.png; do
		echo $i
		convert output/Shadow/`basename $i` $i -composite -background 'rgb(0,255,0)' -alpha remove output/Combined/`basename $i`
	done
}

# Go through the combined frames and re-generate a palette based off all the data
generate_palette() {
	pushd output/Combined
	paletter2

	popd
}

# Go through each of the bamd files and create the bams
generate_bams() {
	for i in Mesh/*.bamd; do
		fname=`basename $i`
		cp $i output/Combined/$fname
		pushd output/Combined
		bammer -input=$fname -output=../`basename $i .bamd`.bam -mode=bam
		popd
	done
}

convert_shadows
convert_frames
overlay_frames_on_shadows
generate_palette
generate_bams
